;===========================================================================================
; KippenLights special edition with PIC16C54
; version 0.2 date 2005-01-01
; Copyright (C) 2004 Stefan Schuermans <1stein@schuermans.info>
; Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
;===========================================================================================

		device		PIC16C54

		config		CP=OFF,WDT=ON,OSC=XT

		include		"PIC16C54.inc"



;===========================================================================================
; constants
;===========================================================================================



;===========================================================================================
; variables
;===========================================================================================

		org		0x07

; general purpose registers
A_REG		ds		1
B_REG		ds		1
C_REG		ds		1
D_REG		ds		1

; number of effects shown
COUNT		ds		1

; number of current effect
EFFECT		ds		1

; current brightness value being output (1..63)
VALUE		ds		1

; brightness values of pixels (0..63)
PIXELS		ds		18



;===========================================================================================
; code entry point
;===========================================================================================

ENTRY:		org		0x0000

		goto		INIT



;===========================================================================================
; interrupt entry point
;===========================================================================================

INTERRUPT:	org		0x0004

		goto		INTERRUPT



;===========================================================================================
; program code
;===========================================================================================

CODE:		org		0x0008

;-------------------------------------------------------------------------------------------
; initialization
;-------------------------------------------------------------------------------------------
INIT:
; set default values for output pins
		movlw		B'11111111'
		movwf		PORTA
		movlw		B'00000000'
		movwf		PORTB
; define inputs/outputs
		movlw		B'00001000'		; RA3: input, rest output
		TRIS		PORTA
		movlw		B'00000000'		; all output
		TRIS		PORTB
; switch timer 0 to internal clock (about 4kHz at 4MHz XTAL)
		movlw		B'11000111'
		OPTION

; reset watchdog
		clrwdt
; initialization done, start program
		goto		MAIN

;-------------------------------------------------------------------------------------------
; set pixels
; INPUT:	W		brightness values to set pixels to
; MODIFIES:	PIXELS
;-------------------------------------------------------------------------------------------
SETPIX:

		movwf		PIXELS+0
		movwf		PIXELS+1
		movwf		PIXELS+2
		movwf		PIXELS+3
		movwf		PIXELS+4
		movwf		PIXELS+5
		movwf		PIXELS+6
		movwf		PIXELS+7
		movwf		PIXELS+8
		movwf		PIXELS+9
		movwf		PIXELS+10
		movwf		PIXELS+11
		movwf		PIXELS+12
		movwf		PIXELS+13
		movwf		PIXELS+14
		movwf		PIXELS+15
		movwf		PIXELS+16
		movwf		PIXELS+17

		retlw		0

;-------------------------------------------------------------------------------------------
; output a frame
; INPUT:	PIXELS		brightness values of pixels
; MODIFIES:	PORTA, PORTB, A_REG, VALUE
; 7250 cycles (with call and retlw)
;-------------------------------------------------------------------------------------------
OUTPUT:

; output frame - start - 7246 cycles

; do an output step for every brightness value from 63 to 1
		movlw		63
		movwf		VALUE
OUTPUT_FRAME_LOOP:

; output step - start - 112 cycles

; output bits for current brightness value

; get bits for 1st 6 LEDs (25 cycles)
		clrf		A_REG

		movf		VALUE,W
		subwf		PIXELS+0,W		; C = (PIXELS+0 >= VALUE)
		btfsc		STATUS,C
		bsf		A_REG,0			; A_REG,0 = C

		movf		VALUE,W
		subwf		PIXELS+1,W		; C = (PIXELS+1 >= VALUE)
		btfsc		STATUS,C
		bsf		A_REG,1			; A_REG,1 = C

		movf		VALUE,W
		subwf		PIXELS+2,W		; C = (PIXELS+2 >= VALUE)
		btfsc		STATUS,C
		bsf		A_REG,2			; A_REG,2 = C

		movf		VALUE,W
		subwf		PIXELS+3,W		; C = (PIXELS+3 >= VALUE)
		btfsc		STATUS,C
		bsf		A_REG,3			; A_REG,3 = C

		movf		VALUE,W
		subwf		PIXELS+4,W		; C = (PIXELS+4 >= VALUE)
		btfsc		STATUS,C
		bsf		A_REG,4			; A_REG,4 = C

		movf		VALUE,W
		subwf		PIXELS+5,W		; C = (PIXELS+5 >= VALUE)
		btfsc		STATUS,C
		bsf		A_REG,5			; A_REG,5 = C

; output bits to 1st 6 LEDs (3 cycles)
		movf		A_REG,W
		movwf		PORTB
		bcf		PORTA,0

; get bits for 2nd 6 LEDs (25 cycles)
		clrf		A_REG

		movf		VALUE,W
		subwf		PIXELS+6,W		; C = (PIXELS+6 >= VALUE)
		btfsc		STATUS,C
		bsf		A_REG,0			; A_REG,0 = C

		movf		VALUE,W
		subwf		PIXELS+7,W		; C = (PIXELS+7 >= VALUE)
		btfsc		STATUS,C
		bsf		A_REG,1			; A_REG,1 = C

		movf		VALUE,W
		subwf		PIXELS+8,W		; C = (PIXELS+8 >= VALUE)
		btfsc		STATUS,C
		bsf		A_REG,2			; A_REG,2 = C

		movf		VALUE,W
		subwf		PIXELS+9,W		; C = (PIXELS+9 >= VALUE)
		btfsc		STATUS,C
		bsf		A_REG,3			; A_REG,3 = C

		movf		VALUE,W
		subwf		PIXELS+10,W		; C = (PIXELS+10 >= VALUE)
		btfsc		STATUS,C
		bsf		A_REG,4			; A_REG,4 = C

		movf		VALUE,W
		subwf		PIXELS+11,W		; C = (PIXELS+11 >= VALUE)
		btfsc		STATUS,C
		bsf		A_REG,5			; A_REG,5 = C

; turn off LEDs (1 cycle)
		bsf		PORTA,0

; output bits to 2nd 6 LEDs (3 cycles)
		movf		A_REG,W
		movwf		PORTB
		bcf		PORTA,1

; get bits for 3rd 6 LEDs (25 cycles)
		clrf		A_REG

		movf		VALUE,W
		subwf		PIXELS+12,W		; C = (PIXELS+12 >= VALUE)
		btfsc		STATUS,C
		bsf		A_REG,0			; A_REG,0 = C

		movf		VALUE,W
		subwf		PIXELS+13,W		; C = (PIXELS+13 >= VALUE)
		btfsc		STATUS,C
		bsf		A_REG,1			; A_REG,1 = C

		movf		VALUE,W
		subwf		PIXELS+14,W		; C = (PIXELS+14 >= VALUE)
		btfsc		STATUS,C
		bsf		A_REG,2			; A_REG,2 = C

		movf		VALUE,W
		subwf		PIXELS+15,W		; C = (PIXELS+15 >= VALUE)
		btfsc		STATUS,C
		bsf		A_REG,3			; A_REG,3 = C

		movf		VALUE,W
		subwf		PIXELS+16,W		; C = (PIXELS+16 >= VALUE)
		btfsc		STATUS,C
		bsf		A_REG,4			; A_REG,4 = C

		movf		VALUE,W
		subwf		PIXELS+17,W		; C = (PIXELS+17 >= VALUE)
		btfsc		STATUS,C
		bsf		A_REG,5			; A_REG,5 = C

; turn off LEDs (1 cycle)
		bsf		PORTA,1

; output bits to 3rd 6 LEDs (3 cycles)
		movf		A_REG,W
		movwf		PORTB
		bcf		PORTA,2

; give 3rd 6 LEDs time to light (25 cycles)
		movlw		8
		movwf		A_REG
OUTPUT_STEP_WAIT:
		decfsz		A_REG
		goto		OUTPUT_STEP_WAIT

; turn off LEDs (1 cycle)
		bsf		PORTA,2

; output step - end - 112 cycles

; do an output step for every brightness value from 63 to 1
		decfsz		VALUE
		goto		OUTPUT_FRAME_LOOP

; output frame - end - 7246 cycles

		retlw		0

;-------------------------------------------------------------------------------------------
; main code
;-------------------------------------------------------------------------------------------
MAIN:
; initialize
		clrf		COUNT

; main loop
MAIN_LOOP:

; randomize effect number every 16 effects
		movlw		0x0F
		andwf		COUNT,W
		btfss		STATUS,Z
		goto		MAIN_NO_RANDOM
		movlw		0
		btfsc		PORTA,3
		movlw		3
		addwf		EFFECT
MAIN_NO_RANDOM:
		incf		COUNT

; show next effect
		incf		EFFECT

		btfsc		EFFECT,2
		goto		MAIN_4
		btfsc		EFFECT,1
		goto		MAIN_2
		btfsc		EFFECT,0
		goto		FLICKER
		goto		RUNC
MAIN_2:
		btfsc		EFFECT,0
		goto		BLINK
		goto		WORMCC
MAIN_4:
		btfsc		EFFECT,1
		goto		MAIN_6
		btfsc		EFFECT,0
		goto		WOBBLE
		goto		RUNCC
MAIN_6:
		btfsc		EFFECT,0
		goto		GLOW
		goto		WORMC

;-------------------------------------------------------------------------------------------
; blinking heart
; MODIFIES:	PORTA, PORTB, A_REG, B_REG, C_REG, VALUE, PIXELS
;-------------------------------------------------------------------------------------------
BLINK:

; blink 3 times
		movlw		3
		movwf		C_REG
BLINK_LOOP:

; turn all pixels on
		movlw		63
		call		SETPIX

; output 50 frames
		movlw		50
		movwf		B_REG
BLINK_ON:
		call		OUTPUT
		clrwdt
		decfsz		B_REG
		goto		BLINK_ON

; turn all pixels off
		movlw		0
		call		SETPIX

; output 50 frames
		movlw		50
		movwf		B_REG
BLINK_OFF:
		call		OUTPUT
		clrwdt
		decfsz		B_REG
		goto		BLINK_OFF

; blink 3 times
		decfsz		C_REG
		goto		BLINK_LOOP

; continue with main loop
		goto		MAIN_LOOP

;-------------------------------------------------------------------------------------------
; glowing heart
; MODIFIES:	PORTA, PORTB, A_REG, B_REG, C_REG, VALUE, PIXELS
;-------------------------------------------------------------------------------------------
GLOW:

; glow 3 times
		movlw		3
		movwf		C_REG
GLOW_LOOP:

; increase glowing
		movlw		0
		movwf		B_REG
GLOW_INC:

; set all pixels
		movf		B_REG,W
		call		SETPIX

; output 2 frames
		call		OUTPUT
		clrwdt
		call		OUTPUT
		clrwdt

; increase glowing
		incf		B_REG
		movlw		63
		xorwf		B_REG,W
		btfss		STATUS,Z		; skip if B_REG is 63
		goto		GLOW_INC

; decrease glowing
		movlw		63
		movwf		B_REG
GLOW_DEC:

; set all pixels
		movf		B_REG,W
		call		SETPIX

; output 2 frames
		call		OUTPUT
		clrwdt
		call		OUTPUT
		clrwdt

; decrease glowing
		decfsz		B_REG
		goto		GLOW_DEC

; glow 3 times
		decfsz		C_REG
		goto		GLOW_LOOP

; continue with main loop
		goto		MAIN_LOOP

;-------------------------------------------------------------------------------------------
; flickering heart
; MODIFIES:	PORTA, PORTB, A_REG, B_REG, C_REG, VALUE, PIXELS
;-------------------------------------------------------------------------------------------
FLICKER:

; flicker 10 times
		movlw		10
		movwf		C_REG
FLICKER_LOOP:

; turn odd pixels on
		movlw		63
		clrf		PIXELS+0
		movwf		PIXELS+1
		clrf		PIXELS+2
		movwf		PIXELS+3
		clrf		PIXELS+4
		movwf		PIXELS+5
		clrf		PIXELS+6
		movwf		PIXELS+7
		clrf		PIXELS+8
		movwf		PIXELS+9
		clrf		PIXELS+10
		movwf		PIXELS+11
		clrf		PIXELS+12
		movwf		PIXELS+13
		clrf		PIXELS+14
		movwf		PIXELS+15
		clrf		PIXELS+16
		movwf		PIXELS+17

; output 15 frames
		movlw		15
		movwf		B_REG
FLICKER_ODD:
		call		OUTPUT
		clrwdt
		decfsz		B_REG
		goto		FLICKER_ODD

; turn even pixels on
		movlw		63
		movwf		PIXELS+0
		clrf		PIXELS+1
		movwf		PIXELS+2
		clrf		PIXELS+3
		movwf		PIXELS+4
		clrf		PIXELS+5
		movwf		PIXELS+6
		clrf		PIXELS+7
		movwf		PIXELS+8
		clrf		PIXELS+9
		movwf		PIXELS+10
		clrf		PIXELS+11
		movwf		PIXELS+12
		clrf		PIXELS+13
		movwf		PIXELS+14
		clrf		PIXELS+15
		movwf		PIXELS+16
		clrf		PIXELS+17

; output 15 frames
		movlw		15
		movwf		B_REG
FLICKER_EVEN:
		call		OUTPUT
		clrwdt
		decfsz		B_REG
		goto		FLICKER_EVEN

; flicker 10 times
		decfsz		C_REG
		goto		FLICKER_LOOP

; continue with main loop
		goto		MAIN_LOOP

;-------------------------------------------------------------------------------------------
; wobbling heart
; MODIFIES:	PORTA, PORTB, A_REG, B_REG, C_REG, VALUE, PIXELS
;-------------------------------------------------------------------------------------------
WOBBLE:

; wobble 5 times
		movlw		5
		movwf		D_REG
WOBBLE_LOOP:

; forward
		clrf		B_REG
		movlw		63
		movwf		C_REG
WOBBLE_FW:

; set all pixels
		movf		B_REG,W
		movwf		PIXELS+0
		movwf		PIXELS+2
		movwf		PIXELS+4
		movwf		PIXELS+6
		movwf		PIXELS+8
		movwf		PIXELS+10
		movwf		PIXELS+12
		movwf		PIXELS+14
		movwf		PIXELS+16
		movf		C_REG,W
		movwf		PIXELS+1
		movwf		PIXELS+3
		movwf		PIXELS+5
		movwf		PIXELS+7
		movwf		PIXELS+9
		movwf		PIXELS+11
		movwf		PIXELS+13
		movwf		PIXELS+15
		movwf		PIXELS+17

; output 2 frames
		call		OUTPUT
		clrwdt
		call		OUTPUT
		clrwdt

; forward
		incf		B_REG
		decfsz		C_REG
		goto		WOBBLE_FW

; backward
		movlw		63
		movwf		B_REG
		clrf		C_REG
WOBBLE_BW:

; set all pixels
		movf		B_REG,W
		movwf		PIXELS+0
		movwf		PIXELS+2
		movwf		PIXELS+4
		movwf		PIXELS+6
		movwf		PIXELS+8
		movwf		PIXELS+10
		movwf		PIXELS+12
		movwf		PIXELS+14
		movwf		PIXELS+16
		movf		C_REG,W
		movwf		PIXELS+1
		movwf		PIXELS+3
		movwf		PIXELS+5
		movwf		PIXELS+7
		movwf		PIXELS+9
		movwf		PIXELS+11
		movwf		PIXELS+13
		movwf		PIXELS+15
		movwf		PIXELS+17

; output 2 frames
		call		OUTPUT
		clrwdt
		call		OUTPUT
		clrwdt

; backward
		incf		C_REG
		decfsz		B_REG
		goto		WOBBLE_BW

; wobble 5 times
		decfsz		D_REG
		goto		WOBBLE_LOOP

; continue with main loop
		goto		MAIN_LOOP

;-------------------------------------------------------------------------------------------
; running light - clockwise
; MODIFIES:	PORTA, PORTB, A_REG, B_REG, C_REG, VALUE, PIXELS
;-------------------------------------------------------------------------------------------
RUNC:

; run 5 rounds
		movlw		5
		movwf		D_REG
RUNC_LOOP:

; turn on two pixels opposite to each other clockwise
		clrf		C_REG
RUNC_STEP:

; turn all pixels off
		movlw		0
		call		SETPIX

; turn on C_REG-th and C_REG+9-th pixel
		movlw		PIXELS
                addwf		C_REG,W
		movwf		FSR
		movlw		63
		movwf		IND0
		movlw		9
		addwf		FSR
		movlw		63
		movwf		IND0

; output 20 frames
		movlw		20
		movwf		B_REG
RUNC_OUT:
		call		OUTPUT
		clrwdt
		decfsz		B_REG
		goto		RUNC_OUT

; turn on two pixels opposite to each other clockwise
		incf		C_REG
		movlw		9
		xorwf		C_REG,W
		btfss		STATUS,Z		; skip if C_REG is 9
		goto		RUNC_STEP

; run 5 rounds
		decfsz		D_REG
		goto		RUNC_LOOP

; continue with main loop
		goto		MAIN_LOOP

;-------------------------------------------------------------------------------------------
; running light - couter-clockwise
; MODIFIES:	PORTA, PORTB, A_REG, B_REG, C_REG, VALUE, PIXELS
;-------------------------------------------------------------------------------------------
RUNCC:

; run 5 rounds
		movlw		5
		movwf		D_REG
RUNCC_LOOP:

; turn on two pixels opposite to each other counter-clockwise
		movlw		8
		movwf		C_REG
RUNCC_STEP:

; turn all pixels off
		movlw		0
		call		SETPIX

; turn on C_REG-th and C_REG+9-th pixel
		movlw		PIXELS
                addwf		C_REG,W
		movwf		FSR
		movlw		63
		movwf		IND0
		movlw		9
		addwf		FSR
		movlw		63
		movwf		IND0

; output 20 frames
		movlw		20
		movwf		B_REG
RUNCC_OUT:
		call		OUTPUT
		clrwdt
		decfsz		B_REG
		goto		RUNCC_OUT

; turn on two pixels opposite to each other counter-clockwise
		decf		C_REG
		movlw		-1
		xorwf		C_REG,W
		btfss		STATUS,Z		; skip if C_REG is -1
		goto		RUNCC_STEP

; run 5 rounds
		decfsz		D_REG
		goto		RUNCC_LOOP

; continue with main loop
		goto		MAIN_LOOP

;-------------------------------------------------------------------------------------------
; running worm - clockwise
; MODIFIES:	PORTA, PORTB, A_REG, B_REG, C_REG, VALUE, PIXELS
;-------------------------------------------------------------------------------------------
WORMC:

; turn all pixels off
		movlw		0
		call		SETPIX

; run 3 rounds
		movlw		3
		movwf		D_REG
WORMC_LOOP:

; let worm run one round clockwise
		clrf		C_REG
WORMC_STEP:

; dim down every pixel
		bcf		STATUS,C
		rrf		PIXELS+0
		bcf		STATUS,C
		rrf		PIXELS+1
		bcf		STATUS,C
		rrf		PIXELS+2
		bcf		STATUS,C
		rrf		PIXELS+3
		bcf		STATUS,C
		rrf		PIXELS+4
		bcf		STATUS,C
		rrf		PIXELS+5
		bcf		STATUS,C
		rrf		PIXELS+6
		bcf		STATUS,C
		rrf		PIXELS+7
		bcf		STATUS,C
		rrf		PIXELS+8
		bcf		STATUS,C
		rrf		PIXELS+9
		bcf		STATUS,C
		rrf		PIXELS+10
		bcf		STATUS,C
		rrf		PIXELS+11
		bcf		STATUS,C
		rrf		PIXELS+12
		bcf		STATUS,C
		rrf		PIXELS+13
		bcf		STATUS,C
		rrf		PIXELS+14
		bcf		STATUS,C
		rrf		PIXELS+15
		bcf		STATUS,C
		rrf		PIXELS+16
		bcf		STATUS,C
		rrf		PIXELS+17

; turn on C_REG-th pixel
		movlw		PIXELS
                addwf		C_REG,W
		movwf		FSR
		movlw		63
		movwf		IND0

; output 20 frames
		movlw		20
		movwf		B_REG
WORMC_OUT:
		call		OUTPUT
		clrwdt
		decfsz		B_REG
		goto		WORMC_OUT

; let worm run one round clockwise
		incf		C_REG
		movlw		18
		xorwf		C_REG,W
		btfss		STATUS,Z		; skip if C_REG is 18
		goto		WORMC_STEP

; run 3 rounds
		decfsz		D_REG
		goto		WORMC_LOOP

; continue with main loop
		goto		MAIN_LOOP

;-------------------------------------------------------------------------------------------
; running worm - counter-clockwise
; MODIFIES:	PORTA, PORTB, A_REG, B_REG, C_REG, VALUE, PIXELS
;-------------------------------------------------------------------------------------------
WORMCC:

; turn all pixels off
		movlw		0
		call		SETPIX

; run 3 rounds
		movlw		3
		movwf		D_REG
WORMCC_LOOP:

; let worm run one round counter-clockwise
		movlw		17
		movwf		C_REG
WORMCC_STEP:

; dim down every pixel
		bcf		STATUS,C
		rrf		PIXELS+0
		bcf		STATUS,C
		rrf		PIXELS+1
		bcf		STATUS,C
		rrf		PIXELS+2
		bcf		STATUS,C
		rrf		PIXELS+3
		bcf		STATUS,C
		rrf		PIXELS+4
		bcf		STATUS,C
		rrf		PIXELS+5
		bcf		STATUS,C
		rrf		PIXELS+6
		bcf		STATUS,C
		rrf		PIXELS+7
		bcf		STATUS,C
		rrf		PIXELS+8
		bcf		STATUS,C
		rrf		PIXELS+9
		bcf		STATUS,C
		rrf		PIXELS+10
		bcf		STATUS,C
		rrf		PIXELS+11
		bcf		STATUS,C
		rrf		PIXELS+12
		bcf		STATUS,C
		rrf		PIXELS+13
		bcf		STATUS,C
		rrf		PIXELS+14
		bcf		STATUS,C
		rrf		PIXELS+15
		bcf		STATUS,C
		rrf		PIXELS+16
		bcf		STATUS,C
		rrf		PIXELS+17

; turn on C_REG-th pixel
		movlw		PIXELS
                addwf		C_REG,W
		movwf		FSR
		movlw		63
		movwf		IND0

; output 20 frames
		movlw		20
		movwf		B_REG
WORMCC_OUT:
		call		OUTPUT
		clrwdt
		decfsz		B_REG
		goto		WORMCC_OUT

; let worm run one round counter-clockwise
		decf		C_REG
		movlw		-1
		xorwf		C_REG,W
		btfss		STATUS,Z		; skip if C_REG is -1
		goto		WORMCC_STEP

; run 3 rounds
		decfsz		D_REG
		goto		WORMCC_LOOP

; continue with main loop
		goto		MAIN_LOOP



;===========================================================================================
; end of file
;===========================================================================================

		end

